//*******************************************************************************
//* Dieses Programm wurde von Ulrich Radig fr Atmel AVR C geschrieben 	 	*
//* 																			*
//* Ich wnsche jeden viel Spa mit diesen kleinen Web-Server der ohne 			*
//* zustzlichen Speicher auskommt.												*
//*																				*
//* Bei Fragen und Verbesserungen wendet euch per EMail an						*
//*																				*
//* mail@ulrichradig.de															*
//*																				*
//* oder im Forum meiner Web Page : www.ulrichradig.de							*
//*																				*
//*******************************************************************************

//Prototype
void WriteRTL (char,char);
unsigned char ReadRTL (char);
void Init_Realtek_Network_Card (void);
void Write_Ethernet_Frame (char *,unsigned int);
void Read_Ethernet_Frame (char *,unsigned int *);

//Ethernet Card Ports
#define DATA_CTRL_RLT  			DDRA
#define DATA_ADDR_RLT			DDRC	//Fr ATMega128/32 wird bei ATMega102 nicht bentigt
#define ADDR_PORT_RLT  			PORTC
#define DATA_PORT_RLT_WRITE  	PORTA
#define DATA_PORT_RLT_READ 		PINA	//Read
#define WRITE_PIN				6		//gibt den Pin an wo der WriteIO Signal angeschlossen ist
#define READ_PIN				5		//gibt den Pin an wo der ReadIO Signal angeschlossen ist
#define RESET_PIN				7		//gibt den Pin an wo der Reset angeschlossen ist
#define INPUT					0x00	//setzt einen Port auf Input
#define OUTPUT					0xff	//setzt einen Port auf Output

//Ethernet Header Layout
#define	ENETPACKETDEST0			0x00	//MAC Zieladresse 6 Byte (MSB)  
#define	ENETPACKETDEST1			0x01	//MAC Zieladresse 
#define	ENETPACKETDEST2			0x02	//MAC Zieladresse 
#define	ENETPACKETDEST3			0x03	//MAC Zieladresse 
#define	ENETPACKETDEST4			0x04	//MAC Zieladresse 
#define	ENETPACKETDEST5			0x05	//MAC Zieladresse (LSB)
#define	ENETPACKETSRC0			0x06	//MAC Quelladresse 6 Byte (MSB)  
#define	ENETPACKETSRC1			0x07	//MAC Quelladresse
#define	ENETPACKETSRC2			0x08	//MAC Quelladresse
#define	ENETPACKETSRC3			0x09	//MAC Quelladresse
#define	ENETPACKETSRC4			0x0A	//MAC Quelladresse
#define	ENETPACKETSRC5			0x0B	//MAC Quelladresse (LSB)
#define	ENETPACKETTYPE0			0x0C	//2 Byte Packettyp
#define	ENETPACKETTYPE1			0x0D	//Nutzlast 0x0800=IP Datagramm;0x0806 = ARP 
#define ENETPACKETDATA			0x0E	//Anfang von den Nutzlast Data

//ARP Layout
#define	ARP_HWTYPE			  	0x0E	//Hardware Typ enthlt den Code fr Ethernet oder andere Link Layer
#define	ARP_PRTYPE				0x10	//Protokoll Typ enthlt den Code fr IP o. anderes bertragungsprotokoll
#define	ARP_HWLEN			  	0x12	//Lnge der Hardwareadresse enthlt 6 fr 6 Byte lange MAC Adressen
#define	ARP_PRLEN			  	0x13	//Lnge der Protocolladresse enthlt 4 fr 4 Byte lange IP Adressen
#define	ARP_OP				  	0x14	//Enthlt Code der signalisiert ob es sich um eine Anfrage o. Antwort handelt
#define	ARP_SHADDR			  	0x16	//Enthlt die MAC Adresse des Anfragenden   
#define	ARP_SIPADDR			  	0x1C	//Enthlt die IP Adresse des Absenders
#define	ARP_THADDR			  	0x20	//MAC Adresse Ziel, ist in diesem Fall 6 * 00,da die Adresse erst noch herausgefunden werden soll (ARP Request)  
#define	ARP_TIPADDR				0x26	//IP Adresse enthlt die IP Adresse zu der die Kommunikation aufgebaut werden soll 

//IP Header Layout
#define	IP_VERS_LEN				0x0E	//4 BIT Die Versionsnummer von IP, meistens also 4 + 4Bit Headergre 	
#define	IP_TOS					0x0F	//Type of Service
#define	IP_PKTLEN				0x10	//16 Bit Komplette Lng des IP Datagrams in Bytes
#define	IP_ID					0x12	//ID des Packet fr Fragmentierung oder Reassemblierung	
#define	IP_FRAG_OFFSET			0x14	//wird benutzt um ein fragmentiertes IP Packet wieder korrekt zusammenzusetzen	
#define	IP_TTL					0x16	//8 Bit Time to Live die lebenszeit eines Paket
#define	IP_PROTO				0x17	//Zeigt das hherschichtige Protokoll an (TCP, UDP, ICMP)	
#define	IP_HDR_CKSUM			0x18	//Checksumme des IP Headers	
#define	IP_SRCADDR				0x1A	//32 Bit IP Quelladresse
#define	IP_DESTADDR				0x1E	//32 Bit IP Zieladresse
#define	IP_DATA					0x22	//beginn der IP Daten (hier fngt dann UDP TCP oder ICMP an)	

//IP Protocol Types
#define	PROT_ICMP				0x01	//zeigt an die Nutzlasten enthalten das ICMP Protokoll
#define	PROT_TCP				0x06	//zeigt an die Nutzlasten enthalten das TCP Protokoll
#define	PROT_UDP				0x11	//zeigt an die Nutzlasten enthalten das UDP Protokoll

//ICMP Header
#define	ICMP_TYPE				0x22	//8 bit typ identifiziert Aufgabe der Nachricht 0 = Echo Reply 8 = Echo Request
#define	ICMP_CODE				0x23	//8 Bit Code enthlt Detailangaben zu bestimmten Typen
#define	ICMP_CKSUM				0x24	//16 Bit Prfsumme enthlt die CRC Prfsumme
#define	ICMP_ID					0x26	//1 byte Identifizierung
#define	ICMP_SEQNUM				0x28	//Sequenznummer
#define ICMP_DATA				0x2A	//ab hier knnen sich Testdaten befinden (Echo/Request)

//TCP Header Layout
#define	TCP_SRCPORT				0x22	//der Quellport fr die Verbindung (Socket)
#define	TCP_DESTPORT			0x24	//der Zielport fr die Verbindung (Socket)
#define	TCP_SEQNUM				0x26	//numerierter Bytestrom
#define	TCP_ACKNUM				0x2A	//numerierter Bytestrom (Gegenrichtung)
#define	TCP_HDRFLAGS			0x2E	//4Bit Lnge des Headers in 32 Bit schritten rest Flags
#define	TCP_WINDOW				0x30	//Fenstergre max. 64K Byte
#define	TCP_CHKSUM				0x32	//Enthlt eine Prfsumme ber Header und Datenbereich
#define	TCP_URGENTPTR			0x34	//16 Bit fr dringende bertragung
#define TCP_DATA				0x36	//Ab hier beginnt der Datenbereich

//TCP Flags
#define  FIN_FLAG				0x01	//Datenbertragung beendet (finished)
#define  SYN_FLAG				0x02	//Verbindungs aufbauen (synchronize)
#define  RST_FLAG				0x04	//Verbindung neu initialisieren (reset)
#define  PSH_FLAG				0x08	//Datenbergabe an Anwendung beschleunigen (push)
#define  ACK_FLAG				0x10	//Datenbertragung besttigen (acknowledged)

//Realtek Netzwerkkarten Register
#define CR						0x00
#define PSTART					0x01
#define PAR0					0x01                                          
#define PSTOP					0x02
#define BNRY					0x03
#define TPSR					0x04
#define TBCR0					0x05
#define TBCR1					0x06
#define ISR						0x07
#define CURR					0x07                                       
#define RSAR0					0x08
#define CRDA0					0x08
#define RSAR1					0x09	
#define CRDAl					0x09
#define RBCR0					0x0A
#define RBCR1					0x0B
#define RSR						0x0C
#define RCR						0x0C
#define TCR						0x0D
#define CNTR0					0x0D
#define DCR						0x0E
#define CNTR1					0x0E
#define IMR						0x0F
#define CNTR2					0x0F
#define RDMAPORT				0x10
#define RSTPORT					0x18
//RTL8019AS initial register Werte
#define RCRVAL					0x04
#define TCRVAL					0x00
#define DCRVAL					0x58
//PXR and OVW interrupt enabled
#define IMRVAL					0x11
#define TXSTART					0x40
#define RXSTART					0x46
#define RXSTOP					0x60
//RTL8019AS ISR Register Bits
#define RST						7
#define RDC						6
#define OVW						4
#define PRX						0

